#! /bin/sh
#
# mouse:~ppr/src/makeprogs/scriptfixup.sh
# Copyright 1995--2003, Trinity College Computing Center.
# Written by David Chappell.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Last modified 5 August 2003.
#

#
# This shell script is a stream editor which modifies a shell script
# so that certain path definitions will be correct for this installation
# of PPR.  The correct values for these path definitions are taken from
# ../makeprogs/global.mk.
#

# Basic syntax checking:
if [ $# -ne 2 ]
	then
	echo "Usage: scriptfixup <infile> <outfile>"
	exit 1
	fi

# Get the replacement values from paths.sh (which was generated by
# gen_paths.c).
paths_file=`dirname $0`/paths.sh
if [ ! -r "$paths_file" ]
	then
	echo "The file \"$paths_file\" doesn't exist.  You must run make in"
	echo "the makeprogs/ directory."
	exit 1
	fi
. $paths_file

# Extract the information that isn't in paths.sh from global.mk.
USER_PPRWWW=`sed -ne 's/^USER_PPRWWW=\(.*\)$/\1/p' ../Makefile.conf`
EECHO=`sed -ne 's/^EECHO=\(.*\)$/\1/p' ../Makefile.conf`
XWINBINDIR=`sed -ne 's/^XWINBINDIR=\(.*\)$/\1/p' ../Makefile.conf`
SH=`sed -ne 's/^SH=\(.*\)$/\1/p' ../Makefile.conf`
PERL=`sed -ne 's/^PERL=\(.*\)$/\1/p' ../Makefile.conf`
if [ "$SH" = "" ]; then SH="/bin/sh"; fi
if [ "$PERL" = "" ]; then PERL="/usr/bin/perl"; fi

PPR_TCLSH="$HOMEDIR/bin/ppr-tclsh";

# Remove any old version of the target file.
rm -f $2

# Run a shell script thru Sed, changing the program paths
sed \
	-e "s#^\\(\$*\\)HOMEDIR *= *\"[^\"]*\"#\\1HOMEDIR=\"$HOMEDIR\"#" \
	-e "s#^set HOMEDIR *\"[^\"]*\"#set HOMEDIR \"$HOMEDIR\"#" \
	-e "s#^\\(\$*\\)SHAREDIR *= *\"[^\"]*\"#\\1SHAREDIR=\"$SHAREDIR\"#" \
	-e "s#^\\(\$*\\)CONFDIR *= *\"[^\"]*\"#\\1CONFDIR=\"$CONFDIR\"#" \
	-e "s#^\\(\$*\\)VAR_SPOOL_PPR *= *\"[^\"]*\"#\\1VAR_SPOOL_PPR=\"$VAR_SPOOL_PPR\"#" \
	-e "s#^\\(\$*\\)TEMPDIR *= *\"[^\"]*\"#\\1TEMPDIR=\"$TEMPDIR\"#" \
	-e "s#^\\(\$*\\)XWINBINDIR *= *\"[^\"]*\"#\\1XWINBINDIR=\"$XWINBINDIR\"#" \
	-e "s#^\\(\$*\\)EECHO *= *\"[^\"]*\"#\\1EECHO=\"$EECHO\"#" \
	-e "s#^\\(\$*\\)USER_PPR *= *.*#\\1USER_PPR=$USER_PPR#" \
	-e "s#^\\(\$*\\)USER_PPRWWW *= *.*#\\1USER_PPRWWW=$USER_PPRWWW#" \
	-e "s#^\\(\$*\\)GROUP_PPR *= *.*#\\1GROUP_PPR=$GROUP_PPR#" \
	-e "s#^SENDMAIL_PATH=.*#SENDMAIL_PATH=\"$SENDMAIL_PATH\"#" \
	-e "s#^use lib [\"][\?][\"];#use lib \"$HOMEDIR/lib\";#" \
	-e "s@^#! */bin/sh\\( *.*\\)\$@#! $SH\\1@" \
	-e "s@^#! */usr/bin/perl\\( *.*\\)\$@#! $PERL\\1@" \
	-e "s@^#! *ppr-tclsh\\( *.*\\)\$@#! $PPR_TCLSH\\1@" \
	$1 >$2	# infile, outfile

# Make it executable but not writable (so we won't
# edit it by mistake).
chmod 555 $2

exit 0
